﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Batching;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Comm.Enum;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using Swashbuckle.AspNetCore.SwaggerGen;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class BatchingService: IBatchingService ,ITransient
    {
        private readonly ISqlSugarClient _db;
        public BatchingService(ISqlSugarClient db)
        {
            _db = db;
        }
        #region 物料
        /// <summary>
        /// 分页
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetBatchingPage(BatchingListQuery dto)
        {
            #region 查询条件
            List<IConditionalModel> conModels = new List<IConditionalModel>();
            if (!string.IsNullOrEmpty(dto.Name))
            {
                conModels.Add(new ConditionalModel() { FieldName = "Name", ConditionalType = ConditionalType.Like, FieldValue = dto.Name });
            }
            if (!string.IsNullOrEmpty(dto.Type))
            {
                conModels.Add(new ConditionalModel() { FieldName = "Type", ConditionalType = ConditionalType.Equal, FieldValue = dto.Type });
            }
            if (!string.IsNullOrEmpty(dto.StockUint))
            {
                conModels.Add(new ConditionalModel() { FieldName = "StockUint", ConditionalType = ConditionalType.Equal, FieldValue = dto.StockUint });
            }
            #endregion
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_Batching>()
                      .Where(conModels)
                      .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select(a => new BatchingListQuery()
                {
                    Id = a.Id,
                    Name = a.Name,
                    Code = a.Code,
                    StockUint = a.StockUint,
                    CreateAt = a.CreateAt,
                    Type = a.Type.ToString(),
                })
               .ToPageListAsync(dto.Current, dto.PageSize, total);
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = res
            };
            return util;
        }

        public string BatchingTypeEnum(int typeId)
        {
            switch (typeId)
            {
                case 1:
                    return "液体料";
                case 2:
                    return "主料";
                default:
                    return "";
            }
        }

        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddBatching(BatchingInfoDto dto)
        {
            var productCode = await _db.Queryable<BPA_Batching>().FirstAsync(x => x.Code == dto.Code);
            if (productCode != null)
                throw Oops.Oh("编码已存在");
            var productName = await _db.Queryable<BPA_Batching>().FirstAsync(t => t.Name == dto.Name);
            if (productName != null)
                throw Oops.Oh($"{dto.Name}已存在");
            try
            {
                var batching = new BPA_Batching
                {
                    StockUint = dto.StockUint,
                    Code = dto.Code,
                    Name = dto.Name,
                    Type = dto.Type
                };
                var res = await _db.Insertable(batching).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }
        /// <summary>
        /// 修改物料信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateBatching(BatchingInfoDto dto)
        {
            try
            {
                var batching = await _db.Queryable<BPA_Batching>().FirstAsync(t => t.Id == dto.Id);
                if (batching == null)
                    throw Oops.Oh($"{dto.Name}不存在或已删除");
                batching.StockUint = dto.StockUint;
                batching.Name = dto.Name;
                batching.Type = dto.Type;
                var res = await _db.Updateable(batching).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除物料信息
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        public async Task<bool> BatchDelBatching(List<string> Ids)
        {
            try
            {
                var resEntitites = _db.Queryable<BPA_Batching>().In(Ids).ToList();
                var res = await _db.Deleteable(resEntitites).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception)
            {
                throw Oops.Oh("删除失败");
            }
        }
        /// <summary>
        /// 查询所有物料信息
        /// </summary>
        /// <returns></returns>
        public async Task<Dictionary<string, object>> GetBatchingList()
        {
            var res = await _db.Queryable<BPA_Batching>()
               .ToDictionaryAsync(x => x.Id, x => x.Name);
            return res;
        }
        /// <summary>
        /// 查询所有物料信息
        /// </summary>
        /// <returns></returns>
        public async Task<List<BatchingList>> GetBatchingSelectList()
        {
            var res = await _db.Queryable<BPA_Batching>().Select(x=>new BatchingList() { Id=x.Id,Name=x.Name})
               .ToListAsync();
            return res;
        }
        /// <summary>
        /// 更新物料状态
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateBatchingStatus(BatchingStatusDto dto)
        {
            try
            {
                // 查询数据库中是否存在未删除的活动信息
                var resEntitites = _db.Queryable<BPA_Batching>().Where(x => x.Id == dto.Id).First();
                var res =await _db.Updateable(resEntitites).ExecuteCommandAsync();
              
                return res>0;
            }
            catch (Exception)
            {
                return false;
            }

        }
       /// <summary>
       /// 查询物料信息
       /// </summary>
       /// <returns></returns>
        public async Task<List<BatchingView>> GetByTypeBatching()
        {
           
            var item = await _db.Queryable<BPA_Batching>().Select(a => new BatchingView() { Id = a.Id, Name = a.Name }).ToListAsync();
            return item;
        }
        #endregion
        #region 物料类型
        public async Task<List<dynamic>> GetBatchingTypeList()
        {
            var res = await _db.Queryable<BPA_BatchingType>().Select<dynamic>(x => new 
            
            {
                id = x.Id,
                name=x.Name
            }) .ToListAsync();
            return res;
        }
        public async Task<List<dynamic>> GetBatchingTypeList_alm()
        {
            var groupId = App.HttpContext.Request.Headers["groupId"].ToString();
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商id不能为空");
            var res = await _db.Queryable<BPA_BatchingType>().Where(x=>x.GroupId== groupId).Select<dynamic>(x => new

            {
                id = x.Id,
                name = x.Name
            }).ToListAsync();
            return res;
        }
        public async Task<bool> AddBatchingType(BatchingTypeDto dto)
        {
            var productCode = _db.Queryable<BPA_BatchingType>().Where(x => x.Name== dto.Name).ToList();
            if (productCode.Count() > 0)
            {
                throw Oops.Oh("物料类型已存在");
            }
            try
            {
                BPA_BatchingType bPA_Product = new BPA_BatchingType();
                bPA_Product.Name = dto.Name;
                var res = await _db.Insertable(bPA_Product).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception ex)
            {
                throw Oops.Oh("添加失败");
            }
        }
        public async Task<bool> BatchTypeDel(string id)
        {
            try
            {
                // 查询数据库中是否存在未删除的活动信息
                var resEntitites =await _db.Queryable<BPA_Batching>().Where(x=>x.Id== id).FirstAsync();
               var res = await _db.Updateable(resEntitites).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception)
            {
                throw Oops.Oh("删除失败");
            }
        }
        #endregion
        #region 物料单位
        public async Task<List<dynamic>> GetBatchingUintList()
        {
            var res = await _db.Queryable<BPA_BatchingUint>().Select<dynamic>(x => new
            {
                id = x.Id,
                name = x.Name
            }).ToListAsync();
            return res;
        }
        public async Task<List<dynamic>> GetBatchingUintList_alm()
        {
            var groupId = App.HttpContext.Request.Headers["groupId"].ToString();
            if(string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商id不能为空");
            var res = await _db.Queryable<BPA_BatchingUint>().Where(x=>x.GroupId== groupId).Select<dynamic>(x => new
            {
                id = x.Id,
                name = x.Name
            }).ToListAsync();
            return res;
        }
        public async Task<bool> AddBatchingUint(BatchingUintDto dto)
        {
            var productCode = _db.Queryable<BPA_BatchingUint>().Where(x => x.Name == dto.Name).ToList();
            if (productCode.Count() > 0)
            {
                throw Oops.Oh("物料单位已存在");
            }
            try
            {
                BPA_BatchingUint bPA_Product = new BPA_BatchingUint();
                bPA_Product.Name = dto.Name;
                var res = await _db.Insertable(bPA_Product).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception ex)
            {
                throw Oops.Oh("添加失败");
            }
        }
        public async Task<bool> BatchingUintDel(string id)
        {
            try
            {
                // 查询数据库中是否存在未删除的活动信息
                var resEntitites = await _db.Queryable<BPA_BatchingUint>().Where(x => x.Id == id).FirstAsync();
                var res = await _db.Updateable(resEntitites).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception)
            {
                throw Oops.Oh("删除失败");
            }
        }
        #endregion
    }
}
