﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Goods;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class GoodsClassifyService : IGoodsClassifyService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public GoodsClassifyService(ISqlSugarClient db)
        {
            _db = db;
        }

        /// <summary>
        /// 分页获取商品分类
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<GoodsClassifyDataDto>>> GetGoodsClassifyPage(GoodsClassifyDto dto)
        {
            List<IConditionalModel> conModels = new List<IConditionalModel>();
            if (!string.IsNullOrEmpty(dto.Name))
            {
                conModels.Add(new ConditionalModel() { FieldName = "Name", ConditionalType = ConditionalType.Like, FieldValue = dto.Name });
            }
            RefAsync<int> total = 0;
            var data = await _db.Queryable<BPA_GoodsClassify>()
                .Where(conModels)
                .OrderBy(t => t.Sort)
                .Select(t => new GoodsClassifyDataDto
                {
                    Id = t.Id,
                    Name = t.Name,
                    Sort = t.Sort,
                    Remark = t.Remark,
                }).ToPageListAsync(dto.Current, dto.PageSize, total);
            return new PageUtil<List<GoodsClassifyDataDto>>()
            {
                Total = total,
                Data = data
            };
        }

        /// <summary>
        /// 更新商品分类
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGoodsClassify(GoodsClassifyDataDto dto)
        {
            var res = 0;
            var classify = await _db.Queryable<BPA_GoodsClassify>().FirstAsync(t => t.Id == dto.Id);
            if(classify == null)
            {
                var check = await _db.Queryable<BPA_GoodsClassify>().AnyAsync(t => t.Name == dto.Name);
                if (check)
                    throw Oops.Oh("已存在该分类");
                classify = new BPA_GoodsClassify
                {
                    Name = dto.Name,
                    Sort = dto.Sort,
                    Remark = dto.Remark
                };
                res = await _db.Insertable(classify).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
            }
            else
            {
                classify.Name = dto.Name;
                classify.Sort = dto.Sort;
                classify.Remark = dto.Remark;
                res = await _db.Updateable(classify).ExecuteCommandAsync();
            }
            return res > 0;
        }

        /// <summary>
        /// 删除商品分类
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        public async Task<bool> DeleteGoodsClassify(string ids)
        {
            var res = 0;
            try
            {
                _db.Ado.BeginTran();
                var idList = ids.Split(",");
                var classifyList = await _db.Queryable<BPA_GoodsClassify>().Where(t => idList.Contains(t.Id)).ToListAsync();
                var relationList = await _db.Queryable<BPA_GoodsClassifyRelation>().Where(t => idList.Contains(t.ClassifyId)).ToListAsync();
                var classifyDelList = new List<BPA_GoodsClassify>();
                var relationDelList = new List<BPA_GoodsClassifyRelation>();
                foreach (var id in idList)
                {
                    var classify = classifyList.FirstOrDefault(t => t.Id == id);
                    if (classify == null)
                        throw Oops.Oh("商品分类不存在或已删除");
                    classifyDelList.Add(classify);
                    var relation = relationList.Where(t => t.ClassifyId == id).ToList();
                    relationDelList.AddRange(relation);
                }
                await _db.Deleteable(classifyDelList).ExecuteCommandAsync();
                await _db.Deleteable(relationDelList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return res > 0;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 根据分类获取商品
        /// </summary>
        /// <param name="classifyId"></param>
        /// <returns></returns>
        public async Task<List<RelationGoodsIdDto>> GetRelation(string classifyId)
        {
            var result = await _db.Queryable<BPA_GoodsClassifyRelation>().Where(t => t.ClassifyId == classifyId)
                .Select(a => new RelationGoodsIdDto
                {
                    GoodsId = a.GoodsId,
                }).ToListAsync();
            return result;
        }

        /// <summary>
        /// 设置商品的分类
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> SetRelation(RelationDto dto)
        {
            try
            {
                var res = 0;
                _db.Ado.BeginTran();
                await _db.Deleteable<BPA_GoodsClassifyRelation>().Where(t => t.ClassifyId == dto.ClassifyId).ExecuteCommandAsync();
                var relationList = new List<BPA_GoodsClassifyRelation>();
                foreach (var item in dto.GoodsIdList)
                {
                    relationList.Add(new BPA_GoodsClassifyRelation
                    {
                        ClassifyId = dto.ClassifyId,
                        GoodsId = item.GoodsId
                    });
                }
                if (relationList.Count <= 0)
                    return true;
                res = await _db.Insertable(relationList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return res > 0;
            }
            catch(Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
    }
}
